import time
from .loggers import Logs
import sys

class Animate:
    @staticmethod
    def Write(text: str, speed: float = 0.05, logger: Logs | None = None):
        output = ""
        for char in text:
            print(char, end="", flush=True)
            output += char
            if logger:
                logger.Info(text)
            time.sleep(speed)
        print()
        return output
    
    @staticmethod
    def NewWrite(text: str, speed: float = 0.01):
        for char in text:
            sys.stdout.write(char)
            sys.stdout.flush()
            time.sleep(speed)

        return input() 


    @staticmethod
    def Wave(text: str, delay: float = 0.1, logger: Logs | None = None) -> str:
        output = ""
        lines = text.splitlines()
        for line in lines:
            print(line)
            output += line + "\n"
            if logger:
                logger.Info(line)
            time.sleep(delay)
        return output




import os

def EnableWindowsAnsiSupport():
    if os.name != 'nt':
        return

    import ctypes
    kernel32 = ctypes.windll.kernel32
    handle = kernel32.GetStdHandle(-11)  
    if handle == 0:
        return
    mode = ctypes.c_uint()
    if not kernel32.GetConsoleMode(handle, ctypes.byref(mode)):
        return
    ENABLE_VIRTUAL_TERMINAL_PROCESSING = 0x0004
    new_mode = mode.value | ENABLE_VIRTUAL_TERMINAL_PROCESSING
    kernel32.SetConsoleMode(handle, new_mode)


isWindows = os.name == 'nt'
isCmd = False

if isWindows:
    import platform
    WT = os.environ.get("WT_SESSION")
    PS = os.environ.get("PSModulePath")
    if WT or PS:
        IS_CMD = False 
    else:
        IS_CMD = True   


class ColorUtils:
    ANSI16Colors = {
        "black": 30,
        "red": 31,
        "green": 32,
        "yellow": 33,
        "blue": 34,
        "magenta": 35,
        "cyan": 36,
        "white": 37,
        "brightBlack": 90,
        "brightRed": 91,
        "brightGreen": 92,
        "brightYellow": 93,
        "brightBlue": 94,
        "brightMagenta": 95,
        "brightCyan": 96,
        "brightWhite": 97,
    }

    @staticmethod
    def RgbToAnsiTruecolor(r, g, b):
        return f"\033[38;2;{r};{g};{b}m"

    @staticmethod
    def RgbToAnsi16color(r, g, b):
        colorMap = {
            (0, 0, 0): 30,
            (128, 0, 0): 31,
            (0, 128, 0): 32,
            (128, 128, 0): 33,
            (0, 0, 128): 34,
            (128, 0, 128): 35,
            (0, 128, 128): 36,
            (192, 192, 192): 37,
            (128, 128, 128): 90,
            (255, 0, 0): 91,
            (0, 255, 0): 92,
            (255, 255, 0): 93,
            (0, 0, 255): 94,
            (255, 0, 255): 95,
            (0, 255, 255): 96,
            (255, 255, 255): 97,
        }

        def dist(c1, c2):
            return sum((a - b) ** 2 for a, b in zip(c1, c2))

        nearestColor = min(colorMap.keys(), key=lambda c: dist(c, (r, g, b)))
        code = colorMap[nearestColor]
        return f"\033[{code}m"

    @staticmethod
    def RgbToAnsi(r, g, b):
        if isWindows and isCmd:
            return ColorUtils.RgbToAnsi16color(r, g, b)
        else:
            return ColorUtils.RgbToAnsiTruecolor(r, g, b)

    @staticmethod
    def InterpolateRgb(colors, t):
        total = len(colors) - 1
        i = int(t * total)
        t = (t * total) - i
        if i >= total:
            return colors[-1]
        r1, g1, b1 = colors[i]
        r2, g2, b2 = colors[i + 1]
        return (
            int(r1 + (r2 - r1) * t),
            int(g1 + (g2 - g1) * t),
            int(b1 + (b2 - b1) * t)
        )



from .utils import isCmd, ColorUtils

class Logs:
    def __init__(self, logging: bool = False, filePath: str | None = None):
        self.Logging = logging
        self.FilePath = filePath if logging else None

    def _WriteFile(self, text: str):
        if not self.FilePath:
            return
        with open(self.FilePath, "a", encoding="utf-8") as f:
            f.write(text + "\n")

    def _Color(self, text: str, rgb: tuple):
        if not isCmd:
            return text
        r, g, b = rgb
        return f"{ColorUtils.RgbToAnsi(r, g, b)}{text}\033[0m"

    def _Build(self, name: str, text: str, rgb: tuple):
        message = f"[{name}] > {text}"
        self._WriteFile(message)
        return self._Color(message, rgb)

    def Error(self, text: str):
        return self._Build("ERROR", text, (255, 0, 0))

    def Warning(self, text: str):
        return self._Build("WARNING", text, (255, 165, 0))

    def Info(self, text: str):
        return self._Build("INFO", text, (0, 200, 255))

    def Success(self, text: str):
        return self._Build("SUCCESS", text, (0, 255, 0))

    def Debug(self, text: str):
        return self._Build("DEBUG", text, (180, 180, 180))



from .utils import ColorUtils

class Colors:
    Black       = (0, 0, 0)
    Gray        = (128, 128, 128)

    White       = (9, 9, 9)

    Red         = (255, 0, 0)
    DarkRed     = (139, 0, 0)
    Orange      = (255, 165, 0)
    Yellow      = (255, 255, 0)

    Green       = (0, 255, 0)
    DarkGreen   = (0, 100, 0)

    Blue        = (0, 0, 255)
    LightBlue   = (173, 216, 230)
    Cyan        = (0, 255, 255)
    DarkBlue    = (0, 0, 139)

    Purple      = (128, 0, 128)
    Pink        = (255, 105, 180)

    @staticmethod
    def Add(text: str, rgb: tuple) -> str:
        r, g, b = rgb
        ansiCode = ColorUtils.RgbToAnsi(r, g, b)
        resetCode = "\033[0m"
        return f"{ansiCode}{text}{resetCode}"



from .colors import Colors
from .gradients import Gradients
from .utils import ColorUtils

from .utils import EnableWindowsAnsiSupport
try:
    EnableWindowsAnsiSupport()
except Exception as e:
    print(f"Re install pip lib | pip install .")




from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="colorkat",
    version="1.0.7",
    description="Color library with multiple features including colors, gradients, logging, and animations",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="g4",
    author_email="glpezg@gmail.com",
    packages=find_packages(),
    include_package_data=True,
    python_requires='>=3.6',
    license="MIT",
)
